@@ -44,6 +44,14 @@ class Server:
 
         return wrapper
 
+    @staticmethod
+    def before_first_request(func):
+        middleware_manager.add_before_first_request(func)
+
+    @staticmethod
+    def before_request(func):
+        middleware_manager.add_before_request(func)
+
     def serve_forever(self, poll_interval=0.5):
         raise NotImplementedError
 
@@ -89,12 +97,16 @@ class AsyncPyws(Server):
         middleware = middleware_cls.from_transports(self.loop._transports)
         super(AsyncPyws, self).add_middleware(middleware)
 
+    @staticmethod
+    def after_last_request(func):
+        middleware_manager.add_after_last_request(func)
+
     async def _server_forever(self):
         server = await self.loop.create_server(lambda: WebSocketProtocol(), self.address, self.port)
         async with server:
             await server.serve_forever()
 
-    def serve_forever(self, *args):
+    def serve_forever(self, *args):  # todo, add os.fork() to run in multiprocessing
         self.middleware_manager.run()
         self.loop.create_task(self._server_forever())
         self.loop.run_forever()
